==> podofo-0.10.3.tar.gz: podofo-0.10.3/src/podofo/main/PdfCatalog.cpp <==
/**
 * SPDX-FileCopyrightText: (C) 2022 Francesco Pretto <ceztko@gmail.com>
 * SPDX-License-Identifier: LGPL-2.0-or-later
 * SPDX-License-Identifier: MPL-2.0
 */

#include <podofo/private/PdfDeclarationsPrivate.h>
#include "PdfCatalog.h"
#include "PdfDictionary.h"
#include "PdfDocument.h"
#include <podofo/auxiliary/StreamDevice.h>

using namespace std;
using namespace PoDoFo;

PdfCatalog::PdfCatalog(PdfObject& obj)
    : PdfDictionaryElement(obj) { }

PdfObject* PdfCatalog::GetMetadataObject()
{
    return GetDictionary().FindKey("Metadata");
}

const PdfObject* PdfCatalog::GetMetadataObject() const
{
    return GetDictionary().FindKey("Metadata");
}

PdfObject& PdfCatalog::GetOrCreateMetadataObject()
{
    auto& dict = GetDictionary();
    auto metadata = dict.FindKey("Metadata");
    if (metadata != nullptr)
        return *metadata;

    metadata = &GetDocument().GetObjects().CreateDictionaryObject("Metadata", "XML");
    dict.AddKeyIndirect("Metadata", *metadata);
    return *metadata;
}

string PdfCatalog::GetMetadataStreamValue() const
{
    string ret;
    auto obj = GetDictionary().FindKey("Metadata");
    if (obj == nullptr)
        return ret;

    auto stream = obj->GetStream();
    if (stream == nullptr)
        return ret;

    StringStreamDevice ouput(ret);
    stream->CopyTo(ouput);
    return ret;
}


void PdfCatalog::SetMetadataStreamValue(const string_view& value)
{
    auto& obj = GetOrCreateMetadataObject();
    auto& stream = obj.GetOrCreateStream();
    stream.SetData(value, true);

    // We are writing raw clear text, which is required in most
    // relevant scenarions (eg. PDF/A). Remove any possibly
    // existing filter
    obj.GetDictionary().RemoveKey(PdfName::KeyFilter);

    // Invalidate current metadata
    GetDocument().GetMetadata().Invalidate();
}

PdfObject* PdfCatalog::GetStructTreeRootObject()
{
    return GetDictionary().FindKey("StructTreeRoot");
}

const PdfObject* PdfCatalog::GetStructTreeRootObject() const
{
    return GetDictionary().FindKey("StructTreeRoot");
}

const PdfObject* PdfCatalog::GetMarkInfoObject() const
{
    return GetDictionary().FindKey("MarkInfo");
}

PdfObject* PdfCatalog::GetMarkInfoObject()
{
    return GetDictionary().FindKey("MarkInfo");
}

PdfObject* PdfCatalog::GetLangObject()
{
    return GetDictionary().FindKey("Lang");
}

const PdfObject* PdfCatalog::GetLangObject() const
{
    return GetDictionary().FindKey("Lang");
}

PdfPageMode PdfCatalog::GetPageMode() const
{
    // PageMode is optional; the default value is UseNone
    PdfPageMode thePageMode = PdfPageMode::UseNone;

    auto pageModeObj = GetDictionary().FindKey("PageMode");
    if (pageModeObj != nullptr)
    {
        PdfName pmName = pageModeObj->GetName();
        if (pmName == "UseNone")
            thePageMode = PdfPageMode::UseNone;
        else if (pmName == "UseThumbs")
            thePageMode = PdfPageMode::UseThumbs;
        else if (pmName == "UseOutlines")
            thePageMode = PdfPageMode::UseBookmarks;
        else if (pmName == "FullScreen")
            thePageMode = PdfPageMode::FullScreen;
        else if (pmName == "UseOC")
            thePageMode = PdfPageMode::UseOC;
        else if (pmName == "UseAttachments")
            thePageMode = PdfPageMode::UseAttachments;
        else
            PODOFO_RAISE_ERROR(PdfErrorCode::InvalidName);
    }

    return thePageMode;
}

void PdfCatalog::SetPageMode(PdfPageMode inMode)
{
    switch (inMode) {
        default:
        case PdfPageMode::DontCare:
            // this value means leave it alone!
            break;

        case PdfPageMode::UseNone:
            GetDictionary().AddKey("PageMode", PdfName("UseNone"));
            break;

        case PdfPageMode::UseThumbs:
            GetDictionary().AddKey("PageMode", PdfName("UseThumbs"));
            break;

        case PdfPageMode::UseBookmarks:
            GetDictionary().AddKey("PageMode", PdfName("UseOutlines"));
            break;

        case PdfPageMode::FullScreen:
            GetDictionary().AddKey("PageMode", PdfName("FullScreen"));
            break;

        case PdfPageMode::UseOC:
            GetDictionary().AddKey("PageMode", PdfName("UseOC"));
            break;

        case PdfPageMode::UseAttachments:
            GetDictionary().AddKey("PageMode", PdfName("UseAttachments"));
            break;
    }
}

void PdfCatalog::SetUseFullScreen()
{
    // first, we get the current mode
    PdfPageMode	curMode = GetPageMode();

    // if current mode is anything but "don't care", we need to move that to non-full-screen
    if (curMode != PdfPageMode::DontCare)
        setViewerPreference("NonFullScreenPageMode", PdfObject(GetDictionary().MustFindKey("PageMode")));

    SetPageMode(PdfPageMode::FullScreen);
}

void PdfCatalog::setViewerPreference(const PdfName& whichPref, const PdfObject& valueObj)
{
    PdfObject* prefsObj = GetDictionary().FindKey("ViewerPreferences");
    if (prefsObj == nullptr)
    {
        // make me a new one and add it
        PdfDictionary vpDict;
        vpDict.AddKey(whichPref, valueObj);
        GetDictionary().AddKey("ViewerPreferences", std::move(vpDict));
    }
    else
    {
        // modify the existing one
        prefsObj->GetDictionary().AddKey(whichPref, valueObj);
    }
}

void PdfCatalog::setViewerPreference(const PdfName& whichPref, bool inValue)
{
    setViewerPreference(whichPref, PdfObject(inValue));
}

void PdfCatalog::SetHideToolbar()
{
    setViewerPreference("HideToolbar", true);
}

void PdfCatalog::SetHideMenubar()
{
    setViewerPreference("HideMenubar", true);
}

void PdfCatalog::SetHideWindowUI()
{
    setViewerPreference("HideWindowUI", true);
}

void PdfCatalog::SetFitWindow()
{
    setViewerPreference("FitWindow", true);
}

void PdfCatalog::SetCenterWindow()
{
    setViewerPreference("CenterWindow", true);
}

void PdfCatalog::SetDisplayDocTitle()
{
    setViewerPreference("DisplayDocTitle", true);
}

void PdfCatalog::SetPrintScaling(const PdfName& scalingType)
{
    setViewerPreference("PrintScaling", scalingType);
}

void PdfCatalog::SetBaseURI(const string_view& inBaseURI)
{
    PdfDictionary uriDict;
    uriDict.AddKey("Base", PdfString(inBaseURI));
    GetDictionary().AddKey("URI", PdfDictionary(uriDict));
}

void PdfCatalog::SetLanguage(const string_view& language)
{
    GetDictionary().AddKey("Lang", PdfString(language));
}

void PdfCatalog::SetBindingDirection(const PdfName& direction)
{
    setViewerPreference("Direction", direction);
}

void PdfCatalog::SetPageLayout(PdfPageLayout layout)
{
    switch (layout)
    {
        default:
        case PdfPageLayout::Ignore:
            break;	// means do nothing
        case PdfPageLayout::Default:
            GetDictionary().RemoveKey("PageLayout");
            break;
        case PdfPageLayout::SinglePage:
            GetDictionary().AddKey("PageLayout", PdfName("SinglePage"));
            break;
        case PdfPageLayout::OneColumn:
            GetDictionary().AddKey("PageLayout", PdfName("OneColumn"));
            break;
        case PdfPageLayout::TwoColumnLeft:
            GetDictionary().AddKey("PageLayout", PdfName("TwoColumnLeft"));
            break;
        case PdfPageLayout::TwoColumnRight:
            GetDictionary().AddKey("PageLayout", PdfName("TwoColumnRight"));
            break;
        case PdfPageLayout::TwoPageLeft:
            GetDictionary().AddKey("PageLayout", PdfName("TwoPageLeft"));
            break;
        case PdfPageLayout::TwoPageRight:
            GetDictionary().AddKey("PageLayout", PdfName("TwoPageRight"));
            break;
    }
}

==> podofo-1.0.2.tar.gz: podofo-1.0.2/src/podofo/main/PdfCatalog.cpp <==
/**
 * SPDX-FileCopyrightText: (C) 2022 Francesco Pretto <ceztko@gmail.com>
 * SPDX-License-Identifier: LGPL-2.0-or-later
 * SPDX-License-Identifier: MPL-2.0
 */

#include <podofo/private/PdfDeclarationsPrivate.h>
#include "PdfCatalog.h"
#include "PdfDictionary.h"
#include "PdfDocument.h"
#include <podofo/auxiliary/StreamDevice.h>

using namespace std;
using namespace PoDoFo;

PdfCatalog::PdfCatalog(PdfObject& obj)
    : PdfDictionaryElement(obj) { }

PdfObject* PdfCatalog::GetMetadataObject()
{
    return GetDictionary().FindKey("Metadata");
}

const PdfObject* PdfCatalog::GetMetadataObject() const
{
    return GetDictionary().FindKey("Metadata");
}

PdfObject& PdfCatalog::GetOrCreateMetadataObject()
{
    auto& dict = GetDictionary();
    auto metadata = dict.FindKey("Metadata");
    if (metadata != nullptr)
        return *metadata;

    metadata = &GetDocument().GetObjects().CreateDictionaryObject("Metadata"_n, "XML"_n);
    dict.AddKeyIndirect("Metadata"_n, *metadata);
    return *metadata;
}

string PdfCatalog::GetMetadataStreamValue() const
{
    string ret;
    auto obj = GetDictionary().FindKey("Metadata");
    if (obj == nullptr)
        return ret;

    auto stream = obj->GetStream();
    if (stream == nullptr)
        return ret;

    StringStreamDevice ouput(ret);
    stream->CopyTo(ouput);
    return ret;
}

void PdfCatalog::SetMetadataStreamValue(const string_view& value)
{
    auto& obj = GetOrCreateMetadataObject();
    auto& stream = obj.GetOrCreateStream();
    stream.SetData(value, true);

    // Invalidate current metadata
    GetDocument().GetMetadata().Invalidate();
}

PdfObject* PdfCatalog::GetStructTreeRootObject()
{
    return GetDictionary().FindKey("StructTreeRoot");
}

const PdfObject* PdfCatalog::GetStructTreeRootObject() const
{
    return GetDictionary().FindKey("StructTreeRoot");
}

const PdfObject* PdfCatalog::GetMarkInfoObject() const
{
    return GetDictionary().FindKey("MarkInfo");
}

PdfObject* PdfCatalog::GetMarkInfoObject()
{
    return GetDictionary().FindKey("MarkInfo");
}

PdfObject* PdfCatalog::GetLangObject()
{
    return GetDictionary().FindKey("Lang");
}

const PdfObject* PdfCatalog::GetLangObject() const
{
    return GetDictionary().FindKey("Lang");
}

PdfPageMode PdfCatalog::GetPageMode() const
{
    // PageMode is optional; the default value is UseNone
    PdfPageMode thePageMode = PdfPageMode::UseNone;

    auto pageModeObj = GetDictionary().FindKey("PageMode");
    if (pageModeObj != nullptr)
    {
        PdfName pmName = pageModeObj->GetName();
        if (pmName == "UseNone")
            thePageMode = PdfPageMode::UseNone;
        else if (pmName == "UseThumbs")
            thePageMode = PdfPageMode::UseThumbs;
        else if (pmName == "UseOutlines")
            thePageMode = PdfPageMode::UseOutlines;
        else if (pmName == "FullScreen")
            thePageMode = PdfPageMode::FullScreen;
        else if (pmName == "UseOC")
            thePageMode = PdfPageMode::UseOC;
        else if (pmName == "UseAttachments")
            thePageMode = PdfPageMode::UseAttachments;
        else
            PODOFO_RAISE_ERROR(PdfErrorCode::InvalidName);
    }

    return thePageMode;
}

void PdfCatalog::SetPageMode(nullable<PdfPageMode> mode)
{
    if (mode == nullptr)
    {
        GetDictionary().RemoveKey("PageMode");
        return;
    }

    switch (*mode)
    {
        case PdfPageMode::UseNone:
            GetDictionary().AddKey("PageMode"_n, "UseNone"_n);
            break;

        case PdfPageMode::UseThumbs:
            GetDictionary().AddKey("PageMode"_n, "UseThumbs"_n);
            break;

        case PdfPageMode::UseOutlines:
            GetDictionary().AddKey("PageMode"_n, "UseOutlines"_n);
            break;

        case PdfPageMode::FullScreen:
            GetDictionary().AddKey("PageMode"_n, "FullScreen"_n);
            break;

        case PdfPageMode::UseOC:
            GetDictionary().AddKey("PageMode"_n, "UseOC"_n);
            break;

        case PdfPageMode::UseAttachments:
            GetDictionary().AddKey("PageMode"_n, "UseAttachments"_n);
            break;
        default:
            PODOFO_RAISE_ERROR(PdfErrorCode::InvalidEnumValue);
    }
}

void PdfCatalog::SetUseFullScreen()
{
    // if current mode is anything but "don't care", we need to move that to non-full-screen
    setViewerPreference("NonFullScreenPageMode"_n, PdfObject(GetDictionary().MustFindKey("PageMode")));

    SetPageMode(PdfPageMode::FullScreen);
}

void PdfCatalog::setViewerPreference(const PdfName& whichPref, const PdfObject& valueObj)
{
    PdfObject* prefsObj = GetDictionary().FindKey("ViewerPreferences");
    if (prefsObj == nullptr)
    {
        // make me a new one and add it
        PdfDictionary vpDict;
        vpDict.AddKey(whichPref, valueObj);
        GetDictionary().AddKey("ViewerPreferences"_n, std::move(vpDict));
    }
    else
    {
        // modify the existing one
        prefsObj->GetDictionary().AddKey(whichPref, valueObj);
    }
}

void PdfCatalog::setViewerPreference(const PdfName& whichPref, bool inValue)
{
    setViewerPreference(whichPref, PdfObject(inValue));
}

void PdfCatalog::SetHideToolbar()
{
    setViewerPreference("HideToolbar"_n, true);
}

void PdfCatalog::SetHideMenubar()
{
    setViewerPreference("HideMenubar"_n, true);
}

void PdfCatalog::SetHideWindowUI()
{
    setViewerPreference("HideWindowUI"_n, true);
}

void PdfCatalog::SetFitWindow()
{
    setViewerPreference("FitWindow"_n, true);
}

void PdfCatalog::SetCenterWindow()
{
    setViewerPreference("CenterWindow"_n, true);
}

void PdfCatalog::SetDisplayDocTitle()
{
    setViewerPreference("DisplayDocTitle"_n, true);
}

void PdfCatalog::SetPrintScaling(const PdfName& scalingType)
{
    setViewerPreference("PrintScaling"_n, scalingType);
}

void PdfCatalog::SetBaseURI(const string_view& inBaseURI)
{
    PdfDictionary uriDict;
    uriDict.AddKey("Base"_n, PdfString(inBaseURI));
    GetDictionary().AddKey("URI"_n, PdfDictionary(uriDict));
}

void PdfCatalog::SetLanguage(const string_view& language)
{
    GetDictionary().AddKey("Lang"_n, PdfString(language));
}

void PdfCatalog::SetBindingDirection(const PdfName& direction)
{
    setViewerPreference("Direction"_n, direction);
}

void PdfCatalog::SetPageLayout(nullable<PdfPageLayout> layout)
{
    if (layout == nullptr)
    {
        GetDictionary().RemoveKey("PageLayout");
        return;
    }

    switch (*layout)
    {
        case PdfPageLayout::SinglePage:
            GetDictionary().AddKey("PageLayout"_n, "SinglePage"_n);
            break;
        case PdfPageLayout::OneColumn:
            GetDictionary().AddKey("PageLayout"_n, "OneColumn"_n);
            break;
        case PdfPageLayout::TwoColumnLeft:
            GetDictionary().AddKey("PageLayout"_n, "TwoColumnLeft"_n);
            break;
        case PdfPageLayout::TwoColumnRight:
            GetDictionary().AddKey("PageLayout"_n, "TwoColumnRight"_n);
            break;
        case PdfPageLayout::TwoPageLeft:
            GetDictionary().AddKey("PageLayout"_n, "TwoPageLeft"_n);
            break;
        case PdfPageLayout::TwoPageRight:
            GetDictionary().AddKey("PageLayout"_n, "TwoPageRight"_n);
            break;
        default:
            PODOFO_RAISE_ERROR(PdfErrorCode::InvalidEnumValue);
    }
}
